//! Types which define the protocol used by the sector layer.

use btlib::{crypto::merkle_stream::VariantMerkleTree, BlockMeta, Inode};
use btproto::protocol;
use btrun::model::CallMsg;

use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct SectorMsg {
    id: SectorId,
    op: SectorOperation,
}

impl CallMsg for SectorMsg {
    type Reply = SectorMsgReply;
}

#[derive(Serialize, Deserialize)]
pub struct SectorId {
    file: FileId,
    sector: SectorKind,
}

#[derive(Serialize, Deserialize)]
pub struct FileId {
    generation: u64,
    inode: Inode,
}

#[derive(Serialize, Deserialize)]
pub enum SectorKind {
    Meta,
    Merkle,
    Data(u64),
}

#[derive(Serialize, Deserialize)]
pub enum SectorOperation {
    Read,
    Write(WriteOperation),
}

#[derive(Serialize, Deserialize)]
pub enum WriteOperation {
    Meta(Box<BlockMeta>),
    Data {
        meta: Box<BlockMeta>,
        contents: Vec<u8>,
    },
}

#[derive(Serialize, Deserialize)]
pub enum SectorMsgReply {
    Meta(BlockMeta),
    Merkle(VariantMerkleTree),
    Data(Vec<u8>),
}

protocol! {
    named SectorProtocol;
    let server = [Listening];
    let client = [Client];
    Client -> Client, >service(Listening)!SectorMsg;
    Listening?SectorMsg -> Listening, >Client!SectorMsg::Reply;
}
